-- Banco de pruebas para la ALU 
-- fichero: TB_ALU.vhd
library IEEE; 
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.ALU_CONSTANTS.all;
use work.TB_ALU_CONSTANTS.all;

entity TB_ALU is 
end entity TB_ALU;

architecture TB_ALU of TB_ALU is
   signal A,B,C : std_logic_vector (WIDTH-1    downto 0);
   signal mode  : std_logic_vector (SEL_BITS-1 downto 0);
  
   component ALU is 
      port ( C    : out  std_logic_vector (WIDTH-1    downto 0);
             A, B : in   std_logic_vector (WIDTH-1    downto 0);
             mode : in   std_logic_vector (SEL_BITS-1 downto 0) );
   end component ALU;
  
   -- Procedure que calcula C (expected_C) y lo compara con el
   -- valor de C que se pasa como argumento (actual_C)
   -- Si ambos valores no coinciden, se muestra un mensaje y se
   -- incrementa el contador de errores (error_count)
   procedure check_ALU ( 
               i, j, k     : in    integer;
               actual_C    : in    std_logic_vector(WIDTH-1 downto 0);
               error_count : inout integer  ) is 
      variable expected_C : integer;
   begin
      case k is
        when 0      => expected_C := i*2;
        when 1      => expected_C := i+j;
        when 2      => expected_C := i-j;
        when 3      => expected_C := -i;
        when 4      => expected_C := TO_INTEGER(signed(
                         std_logic_vector(TO_SIGNED(i,WIDTH)) and  
                         std_logic_vector(TO_SIGNED(j,WIDTH))     ));
        when 5      => expected_C := TO_INTEGER(signed(
                         std_logic_vector(TO_SIGNED(i,WIDTH)) or  
                         std_logic_vector(TO_SIGNED(j,WIDTH))     ));
        when 6      => expected_C := TO_INTEGER(signed(
                         std_logic_vector(TO_SIGNED(i,WIDTH)) xor  
                         std_logic_vector(TO_SIGNED(j,WIDTH))     ));
        when others => expected_C := TO_INTEGER(signed(
                         not std_logic_vector(TO_SIGNED(i,WIDTH)) ));
      end case;
      expected_C := TO_INTEGER(TO_SIGNED(expected_C,WIDTH));
                -- Trunca el resultado a WIDTH bits
      assert( expected_C = TO_INTEGER(signed(actual_C)) )
         report "ERROR. Operandos: " & integer'image(i) & 
                ", " & integer'image(j) &
                ", Operacion: " & integer'image(k) &
                ", resultado esperado: " &
                integer'image(expected_C) &
                ", resultado actual: " &
                integer'image(TO_INTEGER(signed(actual_C))) &
                " en el instante " &
                time'image(now);
      if (expected_C /= TO_INTEGER(signed(actual_C))) then
         error_count := error_count + 1;
      end if;
   end procedure check_ALU;
   -- Fin de la definicin del procedure
  
begin
   UUT : component ALU port map (C, A, B, mode);
  
   -- bloque process para generar los vectores de test y
   -- comprobar el resultado
   main : process is
      variable error_count : integer := 0;
   begin  
      
      report "Comienza la simulacin";
    

      -- Vectores de test: operandos con valor prximo a cero
      for i in -2 to 2 loop
         for j in -2 to 2 loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
               mode <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait for DELAY;
               check_ALU(i, j, k, C, error_count);
            end loop;
         end loop;
      end loop;
    
      -- Vectores de test: operandos con valor prximo al mnimo
      for i in MIN_NEG_DATA to MIN_NEG_DATA+4 loop
         for j in MIN_NEG_DATA to MIN_NEG_DATA+4 loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
               mode <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait for DELAY;
               check_ALU(i, j, k, C, error_count);
            end loop;
         end loop;
      end loop;
    
      -- Vectores de test: operandos con valor prximo al mximo
      for i in MAX_POS_DATA-4 to MAX_POS_DATA loop
         for j in MAX_POS_DATA-4 to MAX_POS_DATA loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
               mode <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait for DELAY;
               check_ALU(i, j, k, C, error_count);
            end loop;
         end loop;
      end loop;

      -- Vectores de test: operandos con valores "al azar"
      for i in 0 to 9 loop
         for j in 0 to 9 loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(41*i-273,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(89*j-384,WIDTH));
               mode <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait for DELAY;
               check_ALU(41*i-273, 89*j-384, k, C, error_count);
            end loop;
         end loop;
      end loop;

      wait for DELAY;
    

      -- Informe mostrando el resultado del test
      if ( error_count = 0 ) then
         report "Finaliza la simulacin: 0 errores";
      else
         report "Finaliza la simulacin: " & 
                integer'image(error_count) & " errores";
      end if;   
        
      wait;  -- Termina la simulacin
   end process main;
   
end architecture TB_ALU;