-- Banco de pruebas para la ALU 
-- fichero: bp_ALUReg.vhd
library IEEE; 
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.ALUReg_CONSTANTS.all;
use work.bp_ALUReg_CONSTANTS.all;

entity bp_ALUReg is 
   constant PERIODO : time := 100 ns;
end entity bp_ALUReg;

architecture bp_ALUReg of bp_ALUReg is

   signal A, B, Z : std_logic_vector (WIDTH-1    downto 0);
   signal Sel     : std_logic_vector (SEL_BITS-1 downto 0);
   signal clk     : std_logic  := '0'; 
  
   component ALUReg is 
      port ( Z    : out std_logic_vector (WIDTH-1    downto 0);
             A, B : in  std_logic_vector (WIDTH-1    downto 0);
             Sel  : in  std_logic_vector (SEL_BITS-1 downto 0);
             clk  : in  std_logic  );
   end component ALUReg;
  
   -- Procedure que calcula Z (expected_Z) y lo compara con el
   -- valor de Z que se pasa como argumento (actual_Z)
   -- Si ambos valores no coinciden, se muestra un mensaje y se
   -- incrementa el contador de errores (error_count)
   procedure check_ALU 
      ( i, j, k     : in    integer;
        actual_Z    : in    std_logic_vector (WIDTH-1 downto 0);
        error_count : inout integer ) is 

      variable var_Z      : integer;
      variable expected_Z : integer;
      variable varAux     : std_logic_vector (WIDTH-1 downto 0);

   begin
      case k is
         when 0 | 1  => var_Z := i+j;
         when 2 | 3  => var_Z := i-j;
         when 4 | 5  => var_Z := TO_INTEGER(signed(
                           std_logic_vector(TO_SIGNED(i,WIDTH)) and  
                           std_logic_vector(TO_SIGNED(j,WIDTH))    ));
         when others => var_Z := TO_INTEGER(signed(
                           std_logic_vector(TO_SIGNED(i,WIDTH)) or  
                           std_logic_vector(TO_SIGNED(j,WIDTH))    ));
      end case;

      case k is
         when 1 | 3 | 5 | 7 => 
            varAUX     := std_logic_vector(TO_SIGNED(var_Z,WIDTH));
            expected_Z := TO_INTEGER(signed(
                             '0' & varAUX(WIDTH-1 downto 1)  ));

         when others        =>   -- Trunca el resultado a WIDTH bits
            expected_Z := TO_INTEGER(TO_SIGNED(var_Z,WIDTH));
      end case;
               
      assert( expected_Z = TO_INTEGER(signed(actual_Z)) )
         report "ERROR. Operandos: "                         & 
                integer'image(i)                             & 
                ", " & integer'image(j)                      &
                ", Operacion: "                              & 
                integer'image(k)                             &
                ", resultado esperado: "                     &
                integer'image(expected_Z)                    &
                ", resultado actual: "                       &
                integer'image(TO_INTEGER(signed(actual_Z)))  &
                " en el instante "                           &
                time'image(now);

      if ( expected_Z /= TO_INTEGER(signed(actual_Z)) ) then
         error_count := error_count + 1;
      end if;
   end procedure check_ALU;
   -- Fin de la definicin del procedure
  
begin
   UUT : component ALUReg port map
         (Z, A, B, Sel, clk);
                  
   clk <= not clk after (PERIODO/2);             

   -- Bloque process para generar los vectores de test y
   -- comprobar el resultado
   main : process is
      variable error_count : integer := 0;
   begin  
      
      report "Comienza la simulacin";
    
      -- Vectores de test: operandos con valor prximo a cero
      for i in -2 to 2 loop
         for j in -2 to 2 loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
               Sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait until falling_edge(clk);
               check_ALU(i, j, k, Z, error_count);
            end loop;
         end loop;
      end loop;
    
      -- Vectores de test: operandos con valor prximo al mnimo
      for i in MIN_NEG_DATA to MIN_NEG_DATA+4 loop
         for j in MIN_NEG_DATA to MIN_NEG_DATA+4 loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
               Sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait until falling_edge(clk);
               check_ALU(i, j, k, Z, error_count);
            end loop;
         end loop;
      end loop;
    
      -- Vectores de test: operandos con valor prximo al mximo
      for i in MAX_POS_DATA-4 to MAX_POS_DATA loop
         for j in MAX_POS_DATA-4 to MAX_POS_DATA loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
               Sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait until falling_edge(clk);
               check_ALU(i, j, k, Z, error_count);
            end loop;
         end loop;
      end loop;

      -- Vectores de test: operandos con valores "al azar"
      for i in 0 to 9 loop
         for j in 0 to 9 loop
            for k in 0 to SEL_MAX loop
               A    <= std_logic_vector(TO_SIGNED(41*i-273,WIDTH));
               b    <= std_logic_vector(TO_SIGNED(89*j-384,WIDTH));
               Sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
               wait until falling_edge(clk);
               check_ALU(41*i-273, 89*j-384, k, Z, error_count);
            end loop;
         end loop;
      end loop;

      wait for PERIODO;
    
      -- Informe mostrando el resultado del test
      if (error_count=0) then
         report "Finaliza la simulacin: 0 errores";
      else
         report "Finaliza la simulacin: " & 
                integer'image(error_count) & 
                " errores";
      end if;   
        
      wait;  -- Termina la simulacin
   end process main;
   
end architecture bp_ALUReg;