-- Banco de pruebas para la mquina de Moore
-- fichero: bp_maquinaMooreSimple.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.STATE_CONSTANTS.all;

entity bp_maquinaMooreSimple is
   constant PERIODO : time := 100 ns; -- Periodo reloj
end entity bp_maquinaMooreSimple;

architecture bp_maquinaMooreSimple of bp_maquinaMooreSimple is
   signal z       : std_logic;   
   signal state   : std_logic_vector(STATE_BITS-1 downto 0);
   signal reset_n : std_logic;   
   signal clk     : std_logic := '0'; -- Inicializada a '0'   
   signal x       : std_logic;  

   -- Declaracin del componente: UUT
   component maquinaMooreSimple is
      port( z       : out std_logic;  -- Seal de salida
            state   : out std_logic_vector(STATE_BITS-1 downto 0);
                                      -- Estado actual de la mquina
            reset_n : in  std_logic;  -- Seal reset activada en bajo
            clk     : in  std_logic;  -- Seal de reloj
            x       : in  std_logic); -- Seal de entrada
   end component maquinaMooreSimple;

   -- Procedimiento para comprobar el resultado
   procedure comprueba_state_z
      ( esperado_state :       std_logic_vector(STATE_BITS-1 downto 0);
        esperado_z     :       std_logic;
        actual_state   :       std_logic_vector(STATE_BITS-1 downto 0);
        actual_z       :       std_logic;
        error_count    : inout integer ) is
   begin
      -- Comprueba el estado
      assert ( esperado_state = actual_state )
         report "ERROR: estado esperado ("         &
                std_logic'image(esperado_state(1)) &
                std_logic'image(esperado_state(0)) &
                "), estado actual ("               &
                std_logic'image(actual_state(1))   &
                std_logic'image(actual_state(0))   &
                ") en el instante "                &
                time'image(now);
      if ( esperado_state /= actual_state ) then
         error_count := error_count + 1;
      end if;
      -- Comprueba la salida
      if ( esperado_z /= actual_z ) then
         report "ERROR: salida esperada ("         &
                std_logic'image(esperado_z)        &
                "), salida actual ("               &
                std_logic'image(actual_z)          &
                ") en el instante "                &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_state_z;
  
begin
   -- Instancia UUT  
   UUT : component maquinaMooreSimple port map
         ( z, state, reset_n, clk, x );
  
   -- Genera un nico pulso LOW para reset_n
   -- En el instante PERIODO/4 se resetea la mquina
   reset_n <= '1',
              '0' after ( PERIODO/4 ),
              '1' after ( PERIODO + PERIODO/4 );
             
   -- Genera seal clk, asumiendo que se inicializ a 0
   -- Flanco de subida en los instantes: 
   -- PERIODO/2, PERIODO+PERIODO/2, 2*PERIODO+PERIODO/2, ...
   clock : clk <= not clk after ( PERIODO/2 );
  
   -- main process: 
   -- genera vectores de test y comprueba resultados
   main : process is
      variable error_count : integer := 0; -- Num. errores
   begin
      report "Comienza la simulacin";
      x <= '0';   -- Valor inicial de entrada a UUT
      wait for PERIODO;
      -- En el instante PERIODO, el estado debera ser S0
      comprueba_state_z(S0, '0', state, z, error_count);
      wait for PERIODO;
      -- En el instante 2*PERIODO, el estado debera ser S2
      comprueba_state_z(S2, '1', state, z, error_count); 
      x <= '1';
      wait for PERIODO;
      -- En el instante 3*PERIODO, el estado debera ser S2
      comprueba_state_z(S2, '1', state, z, error_count);
      x <= '0'; 
      wait for PERIODO;
      -- En el instante 4*PERIODO, el estado debera ser S0
      comprueba_state_z(S0, '0', state, z, error_count); 
      x <= '1';
      wait for PERIODO;    
      -- En el instante 5*PERIODO, el estado debera ser S1
      comprueba_state_z(S1, '0', state, z, error_count); 
      wait for PERIODO;   
      -- En el instante 6*PERIODO, el estado debera ser S1
      comprueba_state_z(S1, '0', state, z, error_count);
      x <= '0';
      wait for PERIODO;  
      -- En el instante 7*PERIODO, el estado debera ser S2
      comprueba_state_z(S2, '1', state, z, error_count); 
      x <= '1'; -- El estado final es S2
      wait for PERIODO;    
      if (error_count = 0) then
         report "Simulacin finalizada con 0 errores";
      else
         report "ERROR: hay "              & 
                integer'image(error_count) &
                " errores.";
      end if;
      wait;   -- Final de la ejecucin del bloque process.
              -- La simulacin continua debido a la sentencia de
              -- asignacin a la seal clk
   end process main;

end architecture bp_maquinaMooreSimple;