-- Banco de pruebas - Mquina detectora secuencia 0110
-- fichero: bp_seq0110.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.seq0110_CONST.all;

entity bp_seq0110 is
   constant PERIODO : time := 100 ns; -- Periodo reloj
end entity bp_seq0110;  

architecture bp_seq0110 of bp_seq0110 is
   signal z_long  : std_logic;         -- Salida de la mquina
   signal state   : std_logic_vector(STATE_BITS-1 downto 0); 
                                       -- Estado actual
   signal reset_n : std_logic;         -- Reset sncrono activado LOW   
   signal clk     : std_logic := '0';  -- Seal de reloj 
   signal x       : std_logic;         -- Entrada a la mquina
    
   -- Declaracin del componente
   component seq0110 is
      port ( z_long  : out std_logic;
             state   : out std_logic_vector(STATE_BITS-1 downto 0); 
             reset_n : in  std_logic;         
             clk     : in  std_logic;         
             x       : in  std_logic );                                    
   end component seq0110;
   
   -- Procedimiento para comprobar si se produce error
   procedure comprueba_estado_salida
      ( esperado_state :       std_logic_vector(STATE_BITS-1 downto 0);
        esperado_z     :       std_logic;
        actual_state   :       std_logic_vector(STATE_BITS-1 downto 0);
        actual_z       :       std_logic;
        error_count    : inout integer) is
   begin
      -- Comprueba el estado
      assert( esperado_state = actual_state ) 
         report "ERROR: Estado esperado ("         &
                std_logic'image(esperado_state(1)) &
                std_logic'image(esperado_state(0)) &
                "), estado actual ("               &
                std_logic'image(actual_state(1))   &
                std_logic'image(actual_state(0))   &
                "), instante: "                    &
                time'image(now);
      if ( esperado_state /= actual_state ) then
         error_count := error_count + 1;
      end if;
       
      -- Comprueba salida
      assert( esperado_z = actual_z ) 
         report "ERROR: Salida esperada ("         &
                std_logic'image(esperado_z)        &
                "), salida actual ("               &
                std_logic'image(actual_z)          &
                "), instante: "                    &
                time'image(now);
      if ( esperado_z /= actual_z ) then
         error_count := error_count + 1;
      end if;
   end procedure comprueba_estado_salida;
   
begin
   -- Instanciar UUT
   UUT: component seq0110 port map
        ( z_long, state, reset_n, clk, x );
         
   -- Generar un pulso LOW en la entrada de reset         
   reset_n <= '1', 
              '0' after ( PERIODO/4 ),
              '1' after ( PERIODO + PERIODO/4 );
   
   -- Generar la seal de reloj, 
   -- asumiendo que ha sido inicializada a '0'
   clk <= not clk after ( PERIODO/2 );
   
   -- Generar vectores de test y comprobar el resultado
   main : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";
      x <= '0';    wait for PERIODO;   -- Comprueba reset, 
                                       -- empieza en S0
      comprueba_estado_salida(S0,'0',state,z_long,error_count); 
      x <= '1';    wait for PERIODO;   -- Sigue en S0
      comprueba_estado_salida(S0,'0',state,z_long,error_count);  
      x <= '0';    wait for PERIODO;   -- Va a S1
      comprueba_estado_salida(S1,'0',state,z_long,error_count); 
      x <= '0';    wait for PERIODO;   -- Sigue en S1
      comprueba_estado_salida(S1,'0',state,z_long,error_count); 
      x <= '1';    wait for PERIODO;   -- Va a S2
      comprueba_estado_salida(S2,'0',state,z_long,error_count); 
      x <= '0';    wait for PERIODO;   -- Va a S1
      comprueba_estado_salida(S1,'0',state,z_long,error_count); 
      x <= '1';    wait for PERIODO;   -- Va a S2
      comprueba_estado_salida(S2,'0',state,z_long,error_count); 
      x <= '1';    wait for PERIODO;   -- Va a S3
      comprueba_estado_salida(S3,'0',state,z_long,error_count); 
      x <= '1';    wait for PERIODO;   -- Va a S0
      comprueba_estado_salida(S0,'0',state,z_long,error_count); 
      x <= '0';    wait for PERIODO;   -- Va a S1
      comprueba_estado_salida(S1,'0',state,z_long,error_count);
      x <= '1';    wait for PERIODO;   -- Va a S2
      comprueba_estado_salida(S2,'0',state,z_long,error_count);
      x <= '1';    wait for PERIODO;   -- Va a S3
      comprueba_estado_salida(S3,'0',state,z_long,error_count);
      x <= '0';    wait for PERIODO/4; -- Va a S1
      comprueba_estado_salida(S3,'0',state,z_long,error_count); 
                                       -- Salida '0' en S3
      wait for 3*(PERIODO/4);         
      comprueba_estado_salida(S1,'1',state,z_long,error_count); 
                                       -- Salida '1' en S1
      wait for PERIODO;                -- Sigue en S1      
      comprueba_estado_salida(S1,'0',state,z_long,error_count);
      wait for PERIODO;                -- Tests finalizados
   
      if (error_count = 0) then
         report "Simulacin finalizada sin errores";
      else
         report  "ERROR: Hay "              & 
                 integer'image(error_count) &
                 " errores.";
      end if;
   
      wait; -- Detiene este bloque process, pero no finaliza la 
            -- simulacin dado que hay asignaciones concurrentes
   end process main;   

end architecture bp_seq0110; 